@@ -12,6 +12,40 @@ namespace Pinta.Core
 {
 	public static class Utility
 	{
+		public static Cairo.Rectangle PointsToRectangle (Cairo.PointD p1, Cairo.PointD p2, bool constrain)
+		{
+			// We want to create a rectangle that always has positive width/height
+			double x, y, w, h;
+
+			if (p1.Y <= p2.Y) {
+				y = p1.Y;
+				h = p2.Y - y;
+			} else {
+				y = p2.Y;
+				h = p1.Y - y;
+			}
+
+			if (p1.X <= p2.X) {
+				x = p1.X;
+
+				if (constrain)
+					w = h;
+				else
+					w = p2.X - x;
+			} else {
+				x = p2.X;
+
+				if (constrain) {
+					w = h;
+					x = p1.X - w;
+				} else
+					w = p1.X - x;
+			}
+
+			return new Cairo.Rectangle (x, y, w, h);
+		}
+
+
 		internal static bool IsNumber (float x)
 		{
 			return x >= float.MinValue && x <= float.MaxValue;
