@@ -33,6 +33,7 @@ using Gdk;
 using Cairo;
 
 using ICSharpCode.SharpZipLib.Zip;
+using System.Collections.Generic;
 
 namespace Pinta.Core
 {
@@ -40,7 +41,7 @@ namespace Pinta.Core
 	{
 		private const int ThumbMaxSize = 256;
 		
-		public void Import (LayerManager layers, string fileName) {
+		public void Import (string fileName) {
 			ZipFile file = new ZipFile (fileName);
 			XmlDocument stackXml = new XmlDocument ();
 			stackXml.Load (file.GetInputStream (file.GetEntry ("stack.xml")));
@@ -48,6 +49,11 @@ namespace Pinta.Core
 			XmlElement imageElement = stackXml.DocumentElement;
 			int width = int.Parse (imageElement.GetAttribute ("w"));
 			int height = int.Parse (imageElement.GetAttribute ("h"));
+
+			Size imagesize = new Size (width, height);
+
+			Document doc = PintaCore.Workspace.CreateAndActivateDocument (fileName, imagesize);
+			doc.HasFile = true;
 			
 			XmlElement stackElement = (XmlElement) stackXml.GetElementsByTagName ("stack")[0];
 			XmlNodeList layerElements = stackElement.GetElementsByTagName ("layer");
@@ -55,11 +61,8 @@ namespace Pinta.Core
 			if (layerElements.Count == 0)
 				throw new XmlException ("No layers found in OpenRaster file");
 
-			layers.Clear ();
-			PintaCore.History.Clear ();
-			layers.DestroySelectionLayer ();
-			PintaCore.Workspace.ImageSize = new Size (width, height);
-			PintaCore.Workspace.CanvasSize = new Gdk.Size (width, height);
+			doc.ImageSize = imagesize;
+			doc.Workspace.CanvasSize = imagesize;
 
 			for (int i = 0; i < layerElements.Count; i++) {
 				XmlElement layerElement = (XmlElement) layerElements[i];
@@ -87,8 +90,9 @@ namespace Pinta.Core
 						}
 					}
 
-					Layer layer = layers.CreateLayer (name, width, height);
-					layers.Insert (layer, 0);
+					Layer layer = doc.CreateLayer (name);
+					doc.Insert (layer, 0);
+
 					layer.Opacity = double.Parse (GetAttribute (layerElement, "opacity", "1"), GetFormat ());
 					
 					using (Pixbuf pb = new Pixbuf (tmp_file)) {
@@ -132,7 +136,7 @@ namespace Pinta.Core
 				return new Size ((int) ((double)width / height * ThumbMaxSize), ThumbMaxSize);
 		}
 
-		private byte[] GetLayerXmlData (LayerManager layers) {
+		private byte[] GetLayerXmlData (List<Layer> layers) {
 			MemoryStream ms = new MemoryStream ();
 			XmlTextWriter writer = new XmlTextWriter (ms, System.Text.Encoding.UTF8);
 			writer.Formatting = Formatting.Indented;
@@ -161,7 +165,8 @@ namespace Pinta.Core
 			return ms.ToArray ();
 		}
 
-		public void Export (LayerManager layers, string fileName) {
+		public void Export (Document document, string fileName)
+		{
 			ZipOutputStream stream = new ZipOutputStream (new FileStream (fileName, FileMode.Create));
 			ZipEntry mimetype = new ZipEntry ("mimetype");
 			mimetype.CompressionMethod = CompressionMethod.Stored;
@@ -170,8 +175,8 @@ namespace Pinta.Core
 			byte[] databytes = System.Text.Encoding.ASCII.GetBytes ("image/openraster");
 			stream.Write (databytes, 0, databytes.Length);
 
-			for (int i = 0; i < layers.Count; i++) {
-				Pixbuf pb = layers[i].Surface.ToPixbuf ();
+			for (int i = 0; i < document.Layers.Count; i++) {
+				Pixbuf pb = document.Layers[i].Surface.ToPixbuf ();
 				byte[] buf = pb.SaveToBuffer ("png");
 				(pb as IDisposable).Dispose ();
 
@@ -180,10 +185,10 @@ namespace Pinta.Core
 			}
 
 			stream.PutNextEntry (new ZipEntry ("stack.xml"));
-			databytes = GetLayerXmlData (layers);
+			databytes = GetLayerXmlData (document.Layers);
 			stream.Write (databytes, 0, databytes.Length);
 
-			ImageSurface flattened = layers.GetFlattenedImage();
+			ImageSurface flattened = document.GetFlattenedImage ();
 			Pixbuf flattenedPb = flattened.ToPixbuf ();
 			Size newSize = GetThumbDimensions (flattenedPb.Width, flattenedPb.Height);
 			Pixbuf thumb = flattenedPb.ScaleSimple (newSize.Width, newSize.Height, InterpType.Bilinear);
