@@ -59,22 +59,26 @@ namespace Pinta
 		internal uint TimerHandle;
 
 		string[] authors = new string[] {
-			"Hanh Pham",
-			"Joe Hillenbrand",
 			"Jonathan Pobst",
-			"Maia Kozheva"
+			"Khairuddin Ni'am",
+			"Mathias Fussenegger",
 		};
 
 		string[] oldAuthors = new string[] {
+			"Aaron Bockover",
 			"Anirudh Sanjeev",
 			"Greg Lowe",
 			"Hanh Pham",
 			"Joe Hillenbrand",
+			"Jon Rimmer",
 			"Jonathan Pobst",
 			"Juergen Obernolte",
 			"Krzysztof Marecki",
+			"Maia Kozheva",
 			"Marco Rolappe",
-			"Olivier Dufour"
+			"Obinou Conseil",
+			"Olivier Dufour",
+			"Richard Cohn"
 		};
 
 		public ScrollBox ()
